classdef JavaRGSolve < handle
% A MATLAB wrapper for the Java rgsolve package which
% implements the Abreu-Sannikov algorithm for solving
% two-player repeated games with perfect monitoring and 
% public randomization.
% 
% See   http://www.princeton.edu/econtheorycenter/research-links/dynamic-games-algorithm/
%       http://www.princeton.edu/~rkatzwer/rgsolve/index.html
%
% Make sure that the file "rgsolve.jar" is in the same directory
% that this MATLAB classdef is placed in so that the appropriate
% Java classes are correctly loaded to the classpath.
%
% Richard M. Katzwer - August 8 2013
% rkatzwer@princeton.edu

    % parameters for the solver
    properties (Access = public)
        maxIters;      % maximum iterations
        convergeTol;   % error tolerance
        algoType;      % algorithm type (Must be 'AS' or 'APS')
        keepActions;   % keeps data on each action 
        keepIters;     % keeps data on each iteration
        doRounding;    % round all quantities? 
        roundDigits;   % number of digits to round to
        outputDigits;  % output digits in V* table
    end
    
    % methods
    methods (Access = public)

        function obj = JavaRGSolve()
        % FUNCTION OBJ = JAVARGSOLVE is a constructor for the rgsolve
        % MATLAB wrapper class.  It instantiates a solver with default
        % parameters (properties).
        
            % initialize some Java stuff
            javaaddpath('rgsolve.jar');
            javax.swing.UIManager.setLookAndFeel('javax.swing.plaf.metal.MetalLookAndFeel');
     
            % defaults
            obj.maxIters = 500;      
            obj.convergeTol = 1e-12;   
            obj.algoType = 'AS';      
            obj.keepActions = 'true';   
            obj.keepIters = 'true';     
            obj.outputDigits = 4;
            obj.doRounding = false;
            obj.roundDigits = 14;
            
        end
        
        function solution = solve( varargin ) 
        % SOLUTION = SOLVE( G1, G2, DELTA, NOTES ) calls rgsolve.jar to 
        % solve the game with payoff matrices G1, G2 and discount DELTA in
        % (0,1). NOTES is an optional string variable describing the game. 
        % Returns an RGSolution Java object as described in the JavaDoc
        %
        % SOLUTION = SOLVE( GAME ) calls rgsolve.jar solver on a Game
        % object GAME from rgsolve.
        %
        % here: http://www.princeton.edu/~rkatzwer/rgsolve/doc/index.html
 
            % set reference to this solver object
            obj = varargin{1};
            
            try
                if nargin == 2
                    if isa( varargin{2}, 'edu.princeton.repeatedgames.rgsolve.games.Game' )
                        game = varargin{2};
                    else 
                         error('Inputs must be as described in method documentation')
                    end

                elseif nargin == 4 || nargin == 5

                     G1 = varargin{2};
                     G2 = varargin{3};
                     delta = varargin{4};
                     if (nargin == 4), notes = ''; else notes = varargin{5}; end
   
                     % instantiate the game in Java as a bimatrix game
                     game = obj.makeGame(G1, G2, delta, notes);

                else
                    error('Inputs must be as described in method documentation') 
                end
            catch
                return;
            end
            
            % set solution parameters
            params = edu.princeton.repeatedgames.rgsolve.AlgoParameters.getDefaults();
            params.maxIters = obj.maxIters;
            params.convergeTol = obj.convergeTol;
            if obj.keepIters; params.keepIters = true; else params.keepIters = false; end
            if obj.keepActions; params.keepActions = true; else params.keepActions = false; end
            if obj.doRounding; params.doRounding = true; else params.doRounding = false; end
            params.roundDigits = obj.roundDigits;
            
            if strcmp(obj.algoType, 'AS')
                params.algo_type = javaMethod('valueOf',...
                'edu.princeton.repeatedgames.rgsolve.AlgoParameters$ALGO_TYPE', ...
                'AS');
            elseif strcmp(obj.algoType, 'APS')
                params.algo_type = javaMethod('valueOf',...
                'edu.princeton.repeatedgames.rgsolve.AlgoParameters$ALGO_TYPE', ...
                'APS');
            else
                error('algorithm type must be ''AS'' or ''APS''')
            end
            
            % solve the game using Java rgsolve package
            solution = edu.princeton.repeatedgames.rgsolve.RGSolve.staticSolveGame(game, params);

            % display solution graphically
            edu.princeton.repeatedgames.rgsolve.components. ...
                SolutionGraph.showSolutionPanel(solution);

            % print out V* information at the end
            V_star_information = showSolutionInfo( obj, solution )

        end
 
        function info = showSolutionInfo( obj, solution )
        % FUNCTION INFO = SHOWSOLUTIONINFO( obj, solution ) returns a table 
        % (in string) form with detailed information about the equilibrium 
        % set V*
            
            info = solution.getSolutionAsText(...
                javaMethod('valueOf',...
                'edu.princeton.repeatedgames.rgsolve.RGSolution$TEXT_OUTPUT_TYPE', ...
                'TEXT_FMT'),...
                obj.outputDigits ...
            );
        end
        
        function Vstar = getExactSolution( obj, solution )
        % FUNCTION VSTAR = GETEXACTSOLUTION( ~, SOLUTION ) accepts an
        % rgsolve RGSolution object and uses the MATLAB symbolic solver to
        % (attempt to) analytically solve the system of equations defining
        % V*.
        
            command = obj.getExactSolutionCommand( solution );
            eval(command);
            Vstar = E;
            
        end
        
        function command = getExactSolutionCommand( ~, solution )
        % FUNCTION COMMAND = GETEXACTSOLUTION( ~, SOLUTION ) accepts an
        % rgsolve RGSolution object as an input and returns a string
        % variable containing a MATLAB command for explicitly solving the
        % system of equations describing the extreme points of V*.
            
            if ~isa( solution, 'edu.princeton.repeatedgames.rgsolve.RGSolution' )
                error('Input must be of type edu.princeton.repeatedgames.rgsolve.RGSolution' )
            end
         
            command = edu.princeton.repeatedgames.rgsolve.RGSolveMathematica.getMatlabCommand(solution.game, solution.V_star, solution.BR1, solution.BR2);

        end
        
        function rggui = RGSolveGUI( varargin )
        % FUNCTION RGGUI = RGSOLVEGUI( VARARGIN ) loads the rgsolve RGGui
        % graphical user interface.  If passed with no arguments, then the
        % GUI is loaded with the standard Abreu-Sannikov 2013 example.  You
        % can, however, load the GUI with a game.  You can pass in an
        % rgolve Game object (Java), or pass an argument list 
        %   RGSolveGUI( G1, G2, DELTA, NOTES)
        % where G1 and G2 are payoff matrices, DELTA is the discount and
        % NOTES contains game notes
            
             % set reference to this solver object
            obj = varargin{1};
            
            try
                if nargin == 1
                    game = [];
         
                elseif nargin == 2
                    if isa( varargin{2}, 'edu.princeton.repeatedgames.rgsolve.games.Game' )
                        game = varargin{2};
                    else 
                         error('Inputs must be as described in method documentation')
                    end

                elseif nargin == 4 || nargin == 5

                     G1 = varargin{2};
                     G2 = varargin{3};
                     delta = varargin{4};
                     if (nargin == 4), notes = ''; else notes = varargin{5}; end
   
                     % instantiate the game in Java as a bimatrix game
                     game = obj.makeGame(G1, G2, delta, notes);

                else
                    error('Inputs must be as described in method documentation') 
                end
            catch
                game = [];
            end
            
            rggui = edu.princeton.repeatedgames.rgsolve.components.RGGui.RGGuiFromMatlab(game);
            
        end
        
        function game = makeGame( ~, G1, G2, delta, notes )
        % FUNCTION GAME = MAKEGAME( ~, G1, G2, DELTA, NOTES ) constructs an
        % rgsolve BimatrixGame object with payoff matrices G1, G2, game
        % notes NOTES and discount factor DELTA.

            % check parameters
            assert( delta > 0 && delta < 1 );
            assert( all(size(G1) == size(G2)) );
            assert( ischar(notes) );

            % instantiate the game in Java as a bimatrix game
            game = edu.princeton.repeatedgames. ...
                rgsolve.games.BimatrixGame( G1, G2, delta, [], notes );

        end

        function V_star = getSolnMatrix( ~, solution ) 
        % FUNCTION V_STAR = GETSOLNMATRIX( ~, SOLUTION ) converts the
        % rgsolve GameExtremePoint array solution.V_star in a MATLAB n x 2
        % matrix, where each row is an extreme point of V*.

            if ~isa( solution, 'edu.princeton.repeatedgames.rgsolve.RGSolution' )
                error('Input must be of type edu.princeton.repeatedgames.rgsolve.RGSolution' )
            end

            numPts = solution.V_star.length;
            V_star = zeros(numPts, 2);
            for i = 1 : numPts
                V_star(i,:) = [solution.V_star(i).x, solution.V_star(i).y];
            end
            
        end
        
        function game = loadGame( ~ )
        % FUNCTION GAME = LOADGAME(~) loads a game from a .mat file and
        % converts it to an rgsolve Game object that
        % can be used in the solver

              try
                  
                  [filename,pathname]=uigetfile('','Select a file');
                  
                  if filename == false
                      game = [];
                  else
                    
                    data = load([pathname filename]);
               
                    G1 = data.G1;
                    G2 = data.G2;
                    delta = data.delta;
                    if isfield(data,'notes'), notes = data.notes; else notes=''; end
                    
                    game = makeGame( G1, G2, delta, notes);
                    
                  end
              catch
                   error('Could not load game')
              end

        end
        
        function didSave = exportGame( ~, G1, G2, delta, notes )
        % FUNCTION DIDSAVE = EXPORTGAME(G1, G2, DELTA, NOTES) Exports a 
        % game to a .mat file that can be read by rgsolve.jar. The payoff 
        % matrices are G1, G2  and discount DELTA is in (0,1). 
        % NOTES is an optional string variable describing the game. 
            
            % check parameters
            assert( delta > 0 && delta < 1 );
            assert( all(size(G1) == size(G2)) );
            assert( ischar(notes) );
            
            % save the game
            try
               [gameName, pathname] = uiputfile('newGame', 'Choose where to Save the Game');
               
               if gameName == 0
               else
                    gameName = strrep(gameName, '.mn', '.mat');
                    save([pathname gameName], 'G1', 'G2', 'delta', 'notes', '-mat');
               end
               
               didSave = true;
            catch
                didSave = false;
            end
            
        end
        
    end
    
end

